@@ -33,9 +33,10 @@ from .itertools import (
     zip_longest,
     groupby,
 )
-from .asynctools import borrow, scoped_iter, await_each, apply, sync
+from .asynctools import borrow, scoped_iter, await_each, any_iter, apply, sync
+from .heapq import merge, nlargest, nsmallest
 
-__version__ = "3.10.2"
+__version__ = "3.10.4"
 
 __all__ = [
     "anext",
@@ -81,6 +82,11 @@ __all__ = [
     "borrow",
     "scoped_iter",
     "await_each",
+    "any_iter",
     "apply",
     "sync",
+    # heapq
+    "merge",
+    "nlargest",
+    "nsmallest",
 ]
